/*\file ControlInputScaleGradient.c
*\brief: update elements properties using an input  vector
*/

#include "./ControlInputScaleGradient.h"

void mexFunction( int nlhs, mxArray* plhs[], int nrhs, const mxArray* prhs[]){

/*input datasets: */
Elements   *elements   = NULL;
Nodes      *nodes      = NULL;
Vertices   *vertices   = NULL;
Loads      *loads      = NULL;
Materials  *materials  = NULL;
Parameters *parameters = NULL;
int         control_type;
double      scaling_factor;

/*Boot module: */
MODULEBOOT();

/*checks on arguments on the matlab side: */
CheckNumMatlabArguments(nlhs,NLHS,nrhs,NRHS,__FUNCT__,&ControlInputScaleGradientUsage);

/*Input datasets: */
FetchData((DataSet**)&elements,ELEMENTSIN);
FetchData((DataSet**)&nodes,NODESIN);
FetchData((DataSet**)&vertices,VERTICESIN);
FetchData((DataSet**)&loads,LOADSIN);
FetchData((DataSet**)&materials,MATERIALSIN);
FetchParams(&parameters,PARAMETERSIN);
FetchData(&control_type,CONTROLTYPE);
FetchData(&scaling_factor,SCALE);

/*configure: */
elements->  Configure(elements,loads, nodes,vertices, materials,parameters);
nodes->     Configure(elements,loads, nodes,vertices, materials,parameters);
loads->     Configure(elements, loads, nodes,vertices, materials,parameters);

/*call "x" code layer*/
ControlInputScaleGradientx(elements,nodes,vertices,loads, materials,parameters,control_type,scaling_factor);

/*write output datasets: */
WriteData(ELEMENTS,elements);
WriteData(NODES,nodes);
WriteData(VERTICES,vertices);
WriteData(LOADS,loads);
WriteData(MATERIALS,materials);
WriteParams(PARAMETERS,parameters);

/*Free ressources: */
delete elements;
delete nodes;
delete vertices;
delete loads;
delete materials;
delete parameters;

/*end module: */
MODULEEND();
}

void ControlInputScaleGradientUsage(void)
{
	_printf_("\n");
	_printf_("   usage: [elements,nodes,vertices,loads,materials,parameters] = %s(elements,nodes,vertices,loads,materials,control_type,scaling_factor);\n",__FUNCT__);
	_printf_("\n");
}
